@@ -374,9 +374,6 @@ extern VideoBootStrap TOOLBOX_bootstrap;
 #if SDL_VIDEO_DRIVER_DRAWSPROCKET
 extern VideoBootStrap DSp_bootstrap;
 #endif
-#if SDL_VIDEO_DRIVER_CYBERGRAPHICS
-extern VideoBootStrap CGX_bootstrap;
-#endif
 #if SDL_VIDEO_DRIVER_PHOTON
 extern VideoBootStrap ph_bootstrap;
 #endif
@@ -395,6 +392,9 @@ extern VideoBootStrap PG_bootstrap;
 #if SDL_VIDEO_DRIVER_DC
 extern VideoBootStrap DC_bootstrap;
 #endif
+#if SDL_VIDEO_DRIVER_NDS
+extern VideoBootStrap NDS_bootstrap;
+#endif
 #if SDL_VIDEO_DRIVER_RISCOS
 extern VideoBootStrap RISCOS_bootstrap;
 #endif
